@@ -1,19 +1,20 @@
 
-import os, time
+import os, time, math
 
 from zope.interface import implements
 from twisted.internet import defer
 import simplejson
-from allmydata.mutable import NotMutableError
+from allmydata.mutable.common import NotMutableError
+from allmydata.mutable.node import MutableFileNode
 from allmydata.interfaces import IMutableFileNode, IDirectoryNode,\
-     IURI, IFileNode, IMutableFileURI, IVerifierURI, IFilesystemNode
-from allmydata.util import hashutil
+     IURI, IFileNode, IMutableFileURI, IVerifierURI, IFilesystemNode, \
+     ExistingChildError
+from allmydata.util import hashutil, mathutil
 from allmydata.util.hashutil import netstring
+from allmydata.util.limiter import ConcurrencyLimiter
 from allmydata.uri import NewDirectoryURI
 from pycryptopp.cipher.aes import AES
 
-from allmydata.mutable import MutableFileNode
-
 def split_netstring(data, numstrings, allow_leftover=False):
     """like string.split(), but extracts netstrings. If allow_leftover=False,
     returns numstrings elements, and throws ValueError if there was leftover
@@ -39,12 +40,85 @@ def split_netstring(data, numstrings, allow_leftover=False):
         raise ValueError("leftover data in netstrings")
     return tuple(elements)
 
+class Deleter:
+    def __init__(self, node, name, must_exist=True):
+        self.node = node
+        self.name = name
+        self.must_exist = True
+    def modify(self, old_contents):
+        children = self.node._unpack_contents(old_contents)
+        if self.name not in children:
+            if self.must_exist:
+                raise KeyError(self.name)
+            self.old_child = None
+            return None
+        self.old_child, metadata = children[self.name]
+        del children[self.name]
+        new_contents = self.node._pack_contents(children)
+        return new_contents
+
+class MetadataSetter:
+    def __init__(self, node, name, metadata):
+        self.node = node
+        self.name = name
+        self.metadata = metadata
+
+    def modify(self, old_contents):
+        children = self.node._unpack_contents(old_contents)
+        children[self.name] = (children[self.name][0], self.metadata)
+        new_contents = self.node._pack_contents(children)
+        return new_contents
+
+
+class Adder:
+    def __init__(self, node, entries=None, overwrite=True):
+        self.node = node
+        if entries is None:
+            entries = []
+        self.entries = entries
+        self.overwrite = overwrite
+
+    def set_node(self, name, node, metadata):
+        self.entries.append( [name, node, metadata] )
+
+    def modify(self, old_contents):
+        children = self.node._unpack_contents(old_contents)
+        now = time.time()
+        for e in self.entries:
+            if len(e) == 2:
+                name, child = e
+                new_metadata = None
+            else:
+                assert len(e) == 3
+                name, child, new_metadata = e
+            assert isinstance(name, unicode)
+            if name in children:
+                if not self.overwrite:
+                    raise ExistingChildError("child '%s' already exists" % name)
+                metadata = children[name][1].copy()
+            else:
+                metadata = {"ctime": now,
+                            "mtime": now}
+            if new_metadata is None:
+                # update timestamps
+                if "ctime" not in metadata:
+                    metadata["ctime"] = now
+                metadata["mtime"] = now
+            else:
+                # just replace it
+                metadata = new_metadata.copy()
+            children[name] = (child, metadata)
+        new_contents = self.node._pack_contents(children)
+        return new_contents
+
 class NewDirectoryNode:
     implements(IDirectoryNode)
     filenode_class = MutableFileNode
 
     def __init__(self, client):
         self._client = client
+        self._most_recent_size = None
+
     def __repr__(self):
         return "<%s %s %s>" % (self.__class__.__name__, self.is_readonly() and "RO" or "RW", hasattr(self, '_uri') and self._uri.abbrev())
     def init_from_uri(self, myuri):
@@ -53,7 +127,7 @@ class NewDirectoryNode:
         self._node.init_from_uri(self._uri.get_filenode_uri())
         return self
 
-    def create(self):
+    def create(self, keypair_generator=None):
         """
         Returns a deferred that eventually fires with self once the directory
         has been created (distributed across a set of storage servers).
@@ -62,15 +136,25 @@ class NewDirectoryNode:
         # URI to create our own.
         self._node = self.filenode_class(self._client)
         empty_contents = self._pack_contents({})
-        d = self._node.create(empty_contents)
+        d = self._node.create(empty_contents, keypair_generator)
         d.addCallback(self._filenode_created)
         return d
     def _filenode_created(self, res):
         self._uri = NewDirectoryURI(IMutableFileURI(self._node.get_uri()))
         return self
 
+    def get_size(self):
+        # return the size of our backing mutable file, in bytes, if we've
+        # fetched it.
+        return self._most_recent_size
+
+    def _set_size(self, data):
+        self._most_recent_size = len(data)
+        return data
+
     def _read(self):
-        d = self._node.download_to_data()
+        d = self._node.download_best_version()
+        d.addCallback(self._set_size)
         d.addCallback(self._unpack_contents)
         return d
 
@@ -200,12 +284,8 @@ class NewDirectoryNode:
         if self.is_readonly():
             return defer.fail(NotMutableError())
         assert isinstance(metadata, dict)
-        d = self._read()
-        def _update(children):
-            children[name] = (children[name][0], metadata)
-            new_contents = self._pack_contents(children)
-            return self._node.update(new_contents)
-        d.addCallback(_update)
+        s = MetadataSetter(self, name, metadata)
+        d = self._node.modify(s.modify)
         d.addCallback(lambda res: self)
         return d
 
@@ -237,7 +317,7 @@ class NewDirectoryNode:
             d.addCallback(_got)
         return d
 
-    def set_uri(self, name, child_uri, metadata=None):
+    def set_uri(self, name, child_uri, metadata=None, overwrite=True):
         """I add a child (by URI) at the specific name. I return a Deferred
         that fires with the child node when the operation finishes. I will
         replace any existing child of the same name.
@@ -248,9 +328,14 @@ class NewDirectoryNode:
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
         assert isinstance(name, unicode)
-        return self.set_node(name, self._create_node(child_uri), metadata)
+        child_node = self._create_node(child_uri)
+        d = self.set_node(name, child_node, metadata, overwrite)
+        d.addCallback(lambda res: child_node)
+        return d
 
-    def set_children(self, entries):
+    def set_children(self, entries, overwrite=True):
+        # this takes URIs
+        a = Adder(self, overwrite=overwrite)
         node_entries = []
         for e in entries:
             if len(e) == 2:
@@ -260,10 +345,10 @@ class NewDirectoryNode:
                 assert len(e) == 3
                 name, child_uri, metadata = e
             assert isinstance(name, unicode)
-            node_entries.append( (name,self._create_node(child_uri),metadata) )
-        return self.set_nodes(node_entries)
+            a.set_node(name, self._create_node(child_uri), metadata)
+        return self._node.modify(a.modify)
 
-    def set_node(self, name, child, metadata=None):
+    def set_node(self, name, child, metadata=None, overwrite=True):
         """I add a child at the specific name. I return a Deferred that fires
         when the operation finishes. This Deferred will fire with the child
         node that was just added. I will replace any existing child of the
@@ -271,48 +356,27 @@ class NewDirectoryNode:
 
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
+
+        if self.is_readonly():
+            return defer.fail(NotMutableError())
         assert isinstance(name, unicode)
         assert IFilesystemNode.providedBy(child), child
-        d = self.set_nodes( [(name, child, metadata)])
+        a = Adder(self, overwrite=overwrite)
+        a.set_node(name, child, metadata)
+        d = self._node.modify(a.modify)
         d.addCallback(lambda res: child)
         return d
 
-    def set_nodes(self, entries):
+    def set_nodes(self, entries, overwrite=True):
         if self.is_readonly():
             return defer.fail(NotMutableError())
-        d = self._read()
-        def _add(children):
-            now = time.time()
-            for e in entries:
-                if len(e) == 2:
-                    name, child = e
-                    new_metadata = None
-                else:
-                    assert len(e) == 3
-                    name, child, new_metadata = e
-                assert isinstance(name, unicode)
-                if name in children:
-                    metadata = children[name][1].copy()
-                else:
-                    metadata = {"ctime": now,
-                                "mtime": now}
-                if new_metadata is None:
-                    # update timestamps
-                    if "ctime" not in metadata:
-                        metadata["ctime"] = now
-                    metadata["mtime"] = now
-                else:
-                    # just replace it
-                    metadata = new_metadata.copy()
-                children[name] = (child, metadata)
-            new_contents = self._pack_contents(children)
-            return self._node.update(new_contents)
-        d.addCallback(_add)
+        a = Adder(self, entries, overwrite=overwrite)
+        d = self._node.modify(a.modify)
         d.addCallback(lambda res: None)
         return d
 
 
-    def add_file(self, name, uploadable, metadata=None):
+    def add_file(self, name, uploadable, metadata=None, overwrite=True):
         """I upload a file (using the given IUploadable), then attach the
         resulting FileNode to the directory at the given name. I return a
         Deferred that fires (with the IFileNode of the uploaded file) when
@@ -323,7 +387,8 @@ class NewDirectoryNode:
         d = self._client.upload(uploadable)
         d.addCallback(lambda results: results.uri)
         d.addCallback(self._client.create_node_from_uri)
-        d.addCallback(lambda node: self.set_node(name, node, metadata))
+        d.addCallback(lambda node:
+                      self.set_node(name, node, metadata, overwrite))
         return d
 
     def delete(self, name):
@@ -332,20 +397,12 @@ class NewDirectoryNode:
         assert isinstance(name, unicode)
         if self.is_readonly():
             return defer.fail(NotMutableError())
-        d = self._read()
-        def _delete(children):
-            old_child, metadata = children[name]
-            del children[name]
-            new_contents = self._pack_contents(children)
-            d = self._node.update(new_contents)
-            def _done(res):
-                return old_child
-            d.addCallback(_done)
-            return d
-        d.addCallback(_delete)
+        deleter = Deleter(self, name)
+        d = self._node.modify(deleter.modify)
+        d.addCallback(lambda res: deleter.old_child)
         return d
 
-    def create_empty_directory(self, name):
+    def create_empty_directory(self, name, overwrite=True):
         """I create and attach an empty directory at the given name. I return
         a Deferred that fires (with the new directory node) when the
         operation finishes."""
@@ -354,14 +411,16 @@ class NewDirectoryNode:
             return defer.fail(NotMutableError())
         d = self._client.create_empty_dirnode()
         def _created(child):
-            d = self.set_node(name, child)
+            entries = [(name, child, None)]
+            a = Adder(self, entries, overwrite=overwrite)
+            d = self._node.modify(a.modify)
             d.addCallback(lambda res: child)
             return d
         d.addCallback(_created)
         return d
 
     def move_child_to(self, current_child_name, new_parent,
-                      new_child_name=None):
+                      new_child_name=None, overwrite=True):
         """I take one of my children and move them to a new parent. The child
         is referenced by name. On the new parent, the child will live under
         'new_child_name', which defaults to 'current_child_name'. I return a
@@ -374,7 +433,8 @@ class NewDirectoryNode:
         assert isinstance(new_child_name, unicode)
         d = self.get(current_child_name)
         def sn(child):
-            return new_parent.set_node(new_child_name, child)
+            return new_parent.set_node(new_child_name, child,
+                                       overwrite=overwrite)
         d.addCallback(sn)
         d.addCallback(lambda child: self.delete(current_child_name))
         return d
@@ -384,12 +444,14 @@ class NewDirectoryNode:
         (directories and files) reachable from this one."""
 
         # this is just a tree-walker, except that following each edge
-        # requires a Deferred.
+        # requires a Deferred. We use a ConcurrencyLimiter to make sure the
+        # fan-out doesn't cause problems.
 
         manifest = set()
         manifest.add(self.get_verifier())
+        limiter = ConcurrencyLimiter(10) # allow 10 in parallel
 
-        d = self._build_manifest_from_node(self, manifest)
+        d = self._build_manifest_from_node(self, manifest, limiter)
         def _done(res):
             # LIT nodes have no verifier-capability: their data is stored
             # inside the URI itself, so there is no need to refresh anything.
@@ -402,8 +464,8 @@ class NewDirectoryNode:
         d.addCallback(_done)
         return d
 
-    def _build_manifest_from_node(self, node, manifest):
-        d = node.list()
+    def _build_manifest_from_node(self, node, manifest, limiter):
+        d = limiter.add(node.list)
         def _got_list(res):
             dl = []
             for name, (child, metadata) in res.iteritems():
@@ -412,12 +474,129 @@ class NewDirectoryNode:
                     manifest.add(verifier)
                     if IDirectoryNode.providedBy(child):
                         dl.append(self._build_manifest_from_node(child,
-                                                                 manifest))
+                                                                 manifest,
+                                                                 limiter))
             if dl:
                 return defer.DeferredList(dl)
         d.addCallback(_got_list)
         return d
 
+    def deep_stats(self):
+        stats = DeepStats()
+        # we track verifier caps, to avoid double-counting children for which
+        # we've got both a write-cap and a read-cap
+        found = set()
+        found.add(self.get_verifier())
+
+        limiter = ConcurrencyLimiter(10)
+
+        d = self._add_deepstats_from_node(self, found, stats, limiter)
+        d.addCallback(lambda res: stats.get_results())
+        return d
+
+    def _add_deepstats_from_node(self, node, found, stats, limiter):
+        d = limiter.add(node.list)
+        def _got_list(children):
+            dl = []
+            dirsize_bytes = node.get_size()
+            dirsize_children = len(children)
+            stats.add("count-directories")
+            stats.add("size-directories", dirsize_bytes)
+            stats.max("largest-directory", dirsize_bytes)
+            stats.max("largest-directory-children", dirsize_children)
+            for name, (child, metadata) in children.iteritems():
+                verifier = child.get_verifier()
+                if verifier in found:
+                    continue
+                found.add(verifier)
+                if IDirectoryNode.providedBy(child):
+                    dl.append(self._add_deepstats_from_node(child, found,
+                                                            stats, limiter))
+                elif IMutableFileNode.providedBy(child):
+                    stats.add("count-files")
+                    stats.add("count-mutable-files")
+                    # TODO: update the servermap, compute a size, add it to
+                    # size-mutable-files, max it into "largest-mutable-file"
+                elif IFileNode.providedBy(child): # CHK and LIT
+                    stats.add("count-files")
+                    size = child.get_size()
+                    stats.histogram("size-files-histogram", size)
+                    if child.get_uri().startswith("URI:LIT:"):
+                        stats.add("count-literal-files")
+                        stats.add("size-literal-files", size)
+                    else:
+                        stats.add("count-immutable-files")
+                        stats.add("size-immutable-files", size)
+                        stats.max("largest-immutable-file", size)
+            if dl:
+                return defer.DeferredList(dl)
+        d.addCallback(_got_list)
+        return d
+
+class DeepStats:
+    def __init__(self):
+        self.stats = {}
+        for k in ["count-immutable-files",
+                  "count-mutable-files",
+                  "count-literal-files",
+                  "count-files",
+                  "count-directories",
+                  "size-immutable-files",
+                  #"size-mutable-files",
+                  "size-literal-files",
+                  "size-directories",
+                  "largest-directory",
+                  "largest-directory-children",
+                  "largest-immutable-file",
+                  #"largest-mutable-file",
+                  ]:
+            self.stats[k] = 0
+        self.histograms = {}
+        for k in ["size-files-histogram"]:
+            self.histograms[k] = {} # maps (min,max) to count
+        self.buckets = [ (0,0), (1,3)]
+        self.root = math.sqrt(10)
+
+    def add(self, key, value=1):
+        self.stats[key] += value
+
+    def max(self, key, value):
+        self.stats[key] = max(self.stats[key], value)
+
+    def which_bucket(self, size):
+        # return (min,max) such that min <= size <= max
+        # values are from the set (0,0), (1,3), (4,10), (11,31), (32,100),
+        # (101,316), (317, 1000), etc: two per decade
+        assert size >= 0
+        i = 0
+        while True:
+            if i >= len(self.buckets):
+                # extend the list
+                new_lower = self.buckets[i-1][1]+1
+                new_upper = int(mathutil.next_power_of_k(new_lower, self.root))
+                self.buckets.append( (new_lower, new_upper) )
+            maybe = self.buckets[i]
+            if maybe[0] <= size <= maybe[1]:
+                return maybe
+            i += 1
+
+    def histogram(self, key, size):
+        bucket = self.which_bucket(size)
+        h = self.histograms[key]
+        if bucket not in h:
+            h[bucket] = 0
+        h[bucket] += 1
+
+    def get_results(self):
+        stats = self.stats.copy()
+        for key in self.histograms:
+            h = self.histograms[key]
+            out = [ (bucket[0], bucket[1], h[bucket]) for bucket in h ]
+            out.sort()
+            stats[key] = out
+        return stats
+
+
 # use client.create_dirnode() to make one of these
 
 
