@@ -11,7 +11,7 @@ from allmydata.util.assertutil import _assert
 from allmydata import codec, hashtree, storage, uri
 from allmydata.interfaces import IDownloadTarget, IDownloader, IFileURI, \
      IDownloadStatus, IDownloadResults
-from allmydata.encode import NotEnoughPeersError
+from allmydata.encode import NotEnoughSharesError
 from pycryptopp.cipher.aes import AES
 
 class HaveAllPeersError(Exception):
@@ -110,7 +110,12 @@ class Output:
 
     def fail(self, why):
         # this is really unusual, and deserves maximum forensics
-        self.log("download failed!", failure=why, level=log.SCARY)
+        if why.check(DownloadStopped):
+            # except DownloadStopped just means the consumer aborted the
+            # download, not so scary
+            self.log("download stopped", level=log.UNUSUAL)
+        else:
+            self.log("download failed!", failure=why, level=log.SCARY)
         self.downloadable.fail(why)
 
     def close(self):
@@ -327,7 +332,7 @@ class SegmentDownloader:
         return d
 
     def _try(self):
-        # fill our set of active buckets, maybe raising NotEnoughPeersError
+        # fill our set of active buckets, maybe raising NotEnoughSharesError
         active_buckets = self.parent._activate_enough_buckets()
         # Now we have enough buckets, in self.parent.active_buckets.
 
@@ -444,6 +449,7 @@ class FileDownloader:
         self._results.timings["cumulative_fetch"] = 0.0
         self._results.timings["cumulative_decode"] = 0.0
         self._results.timings["cumulative_decrypt"] = 0.0
+        self._results.timings["paused"] = 0.0
 
         if IConsumer.providedBy(downloadable):
             downloadable.registerProducer(self, True)
@@ -484,6 +490,7 @@ class FileDownloader:
         if self._paused:
             return
         self._paused = defer.Deferred()
+        self._paused_at = time.time()
         if self._status:
             self._status.set_paused(True)
 
@@ -498,6 +505,8 @@ class FileDownloader:
     def stopProducing(self):
         self.log("Download.stopProducing")
         self._stopped = True
+        paused_for = time.time() - self._paused_at
+        self._results.timings['paused'] += paused_for
         if self._status:
             self._status.set_stopped(True)
             self._status.set_active(False)
@@ -593,7 +602,7 @@ class FileDownloader:
             self._results.timings["peer_selection"] = now - self._started
 
         if len(self._share_buckets) < self._num_needed_shares:
-            raise NotEnoughPeersError
+            raise NotEnoughSharesError
 
         #for s in self._share_vbuckets.values():
         #    for vb in s:
@@ -632,7 +641,7 @@ class FileDownloader:
     def _obtain_validated_thing(self, ignored, sources, name, methname, args,
                                 validatorfunc):
         if not sources:
-            raise NotEnoughPeersError("started with zero peers while fetching "
+            raise NotEnoughSharesError("started with zero peers while fetching "
                                       "%s" % name)
         bucket = sources[0]
         sources = sources[1:]
@@ -644,7 +653,7 @@ class FileDownloader:
             self.log("%s from vbucket %s failed:" % (name, bucket),
                      failure=f, level=log.WEIRD)
             if not sources:
-                raise NotEnoughPeersError("ran out of peers, last error was %s"
+                raise NotEnoughSharesError("ran out of peers, last error was %s"
                                           % (f,))
             # try again with a different one
             return self._obtain_validated_thing(None, sources, name,
@@ -766,7 +775,7 @@ class FileDownloader:
 
     def _activate_enough_buckets(self):
         """either return a mapping from shnum to a ValidatedBucket that can
-        provide data for that share, or raise NotEnoughPeersError"""
+        provide data for that share, or raise NotEnoughSharesError"""
 
         while len(self.active_buckets) < self._num_needed_shares:
             # need some more
@@ -774,7 +783,7 @@ class FileDownloader:
             available_shnums = set(self._share_vbuckets.keys())
             potential_shnums = list(available_shnums - handled_shnums)
             if not potential_shnums:
-                raise NotEnoughPeersError
+                raise NotEnoughSharesError
             # choose a random share
             shnum = random.choice(potential_shnums)
             # and a random bucket that will provide it
@@ -1038,10 +1047,12 @@ class Downloader(service.MultiService):
     name = "downloader"
     MAX_DOWNLOAD_STATUSES = 10
 
-    def __init__(self):
+    def __init__(self, stats_provider=None):
         service.MultiService.__init__(self)
-        self._all_downloads = weakref.WeakKeyDictionary()
-        self._recent_download_status = []
+        self.stats_provider = stats_provider
+        self._all_downloads = weakref.WeakKeyDictionary() # for debugging
+        self._all_download_statuses = weakref.WeakKeyDictionary()
+        self._recent_download_statuses = []
 
     def download(self, u, t):
         assert self.parent
@@ -1050,16 +1061,18 @@ class Downloader(service.MultiService):
         t = IDownloadTarget(t)
         assert t.write
         assert t.close
+
+        if self.stats_provider:
+            self.stats_provider.count('downloader.files_downloaded', 1)
+            self.stats_provider.count('downloader.bytes_downloaded', u.get_size())
+
         if isinstance(u, uri.LiteralFileURI):
             dl = LiteralDownloader(self.parent, u, t)
         elif isinstance(u, uri.CHKFileURI):
             dl = FileDownloader(self.parent, u, t)
         else:
             raise RuntimeError("I don't know how to download a %s" % u)
-        self._all_downloads[dl] = None
-        self._recent_download_status.append(dl.get_download_status())
-        while len(self._recent_download_status) > self.MAX_DOWNLOAD_STATUSES:
-            self._recent_download_status.pop(0)
+        self._add_download(dl)
         d = dl.start()
         return d
 
@@ -1071,11 +1084,14 @@ class Downloader(service.MultiService):
     def download_to_filehandle(self, uri, filehandle):
         return self.download(uri, FileHandle(filehandle))
 
+    def _add_download(self, downloader):
+        self._all_downloads[downloader] = None
+        s = downloader.get_download_status()
+        self._all_download_statuses[s] = None
+        self._recent_download_statuses.append(s)
+        while len(self._recent_download_statuses) > self.MAX_DOWNLOAD_STATUSES:
+            self._recent_download_statuses.pop(0)
 
-    def list_all_downloads(self):
-        return self._all_downloads.keys()
-    def list_active_downloads(self):
-        return [d.get_download_status() for d in self._all_downloads.keys()
-                if d.get_download_status().get_active()]
-    def list_recent_downloads(self):
-        return self._recent_download_status
+    def list_all_download_statuses(self):
+        for ds in self._all_download_statuses:
+            yield ds
