@@ -1,5 +1,5 @@
 
-import os.path, stat, time
+import os, stat, time, weakref
 from zope.interface import implements
 from twisted.application import service
 from twisted.internet import defer
@@ -138,12 +138,15 @@ class CHKUploadHelper(Referenceable, upload.CHKUploader):
         self._helper = helper
         self._incoming_file = incoming_file
         self._encoding_file = encoding_file
-        upload_id = storage.si_b2a(storage_index)[:5]
+        self._upload_id = storage.si_b2a(storage_index)[:5]
         self._log_number = log_number
         self._results = results
         self._upload_status = upload.UploadStatus()
         self._upload_status.set_helper(False)
-        self._helper.log("CHKUploadHelper starting for SI %s" % upload_id,
+        self._upload_status.set_storage_index(storage_index)
+        self._upload_status.set_status("fetching ciphertext")
+        self._upload_status.set_progress(0, 1.0)
+        self._helper.log("CHKUploadHelper starting for SI %s" % self._upload_id,
                          parent=log_number)
 
         self._client = helper.parent
@@ -205,12 +208,16 @@ class CHKUploadHelper(Referenceable, upload.CHKUploader):
         self._reader.close()
         os.unlink(self._encoding_file)
         self._finished_observers.fire(r)
-        self._helper.upload_finished(self._storage_index)
+        self._helper.upload_finished(self._storage_index, size)
         del self._reader
 
     def _failed(self, f):
+        self.log(format="CHKUploadHelper(%(si)s) failed",
+                 si=storage.si_b2a(self._storage_index)[:5],
+                 failure=f,
+                 level=log.UNUSUAL)
         self._finished_observers.fire(f)
-        self._helper.upload_finished(self._storage_index)
+        self._helper.upload_finished(self._storage_index, 0)
         del self._reader
 
 class AskUntilSuccessMixin:
@@ -254,6 +261,7 @@ class CHKCiphertextFetcher(AskUntilSuccessMixin):
         self._upload_helper = helper
         self._incoming_file = incoming_file
         self._encoding_file = encoded_file
+        self._upload_id = helper._upload_id
         self._log_parent = logparent
         self._done_observers = observer.OneShotObserverList()
         self._readers = []
@@ -306,12 +314,14 @@ class CHKCiphertextFetcher(AskUntilSuccessMixin):
 
     def _got_size(self, size):
         self.log("total size is %d bytes" % size, level=log.NOISY)
+        self._upload_helper._upload_status.set_size(size)
         self._expected_size = size
 
     def _start_reading(self, res):
         # then find out how much crypttext we have on disk
         if os.path.exists(self._incoming_file):
             self._have = os.stat(self._incoming_file)[stat.ST_SIZE]
+            self._upload_helper._helper.count("chk_upload_helper.resumes")
             self.log("we already have %d bytes" % self._have, level=log.NOISY)
         else:
             self._have = 0
@@ -356,7 +366,8 @@ class CHKCiphertextFetcher(AskUntilSuccessMixin):
             else:
                 self._loop(fire_when_done)
         def _err(f):
-            self.log("ciphertext read failed", failure=f, level=log.UNUSUAL)
+            self.log(format="[%(si)s] ciphertext read failed",
+                     si=self._upload_id, failure=f, level=log.UNUSUAL)
             fire_when_done.errback(f)
         d.addCallbacks(_done, _err)
         return None
@@ -365,11 +376,17 @@ class CHKCiphertextFetcher(AskUntilSuccessMixin):
         needed = self._expected_size - self._have
         fetch_size = min(needed, self.CHUNK_SIZE)
         if fetch_size == 0:
+            self._upload_helper._upload_status.set_progress(1, 1.0)
             return True # all done
-        self.log(format="fetching %(start)d-%(end)d of %(total)d",
+        percent = 0.0
+        if self._expected_size:
+            percent = 1.0 * (self._have+fetch_size) / self._expected_size
+        self.log(format="fetching [%(si)s] %(start)d-%(end)d of %(total)d (%(percent)d%%)",
+                 si=self._upload_id,
                  start=self._have,
                  end=self._have+fetch_size,
                  total=self._expected_size,
+                 percent=int(100.0*percent),
                  level=log.NOISY)
         d = self.call("read_encrypted", self._have, fetch_size)
         def _got_data(ciphertext_v):
@@ -377,6 +394,8 @@ class CHKCiphertextFetcher(AskUntilSuccessMixin):
                 self._f.write(data)
                 self._have += len(data)
                 self._ciphertext_fetched += len(data)
+                self._upload_helper._helper.count("chk_upload_helper.fetched_bytes", len(data))
+                self._upload_helper._upload_status.set_progress(1, percent)
             return False # not done
         d.addCallback(_got_data)
         return d
@@ -422,10 +441,8 @@ class LocalCiphertextReader(AskUntilSuccessMixin):
         self._encoding_file = encoding_file
         self._status = None
 
-    def set_upload_status(self, upload_status):
-        self._status = interfaces.IUploadStatus(upload_status)
-
     def start(self):
+        self._upload_helper._upload_status.set_status("pushing")
         self._size = os.stat(self._encoding_file)[stat.ST_SIZE]
         self.f = open(self._encoding_file, "rb")
 
@@ -456,7 +473,7 @@ class LocalCiphertextReader(AskUntilSuccessMixin):
 
 
 class Helper(Referenceable, service.MultiService):
-    implements(interfaces.RIHelper)
+    implements(interfaces.RIHelper, interfaces.IStatsProducer)
     # this is the non-distributed version. When we need to have multiple
     # helpers, this object will become the HelperCoordinator, and will query
     # the farm of Helpers to see if anyone has the storage_index of interest,
@@ -465,22 +482,79 @@ class Helper(Referenceable, service.MultiService):
 
     name = "helper"
     chk_upload_helper_class = CHKUploadHelper
+    MAX_UPLOAD_STATUSES = 10
 
-    def __init__(self, basedir):
+    def __init__(self, basedir, stats_provider=None):
         self._basedir = basedir
         self._chk_incoming = os.path.join(basedir, "CHK_incoming")
         self._chk_encoding = os.path.join(basedir, "CHK_encoding")
         fileutil.make_dirs(self._chk_incoming)
         fileutil.make_dirs(self._chk_encoding)
         self._active_uploads = {}
+        self._all_uploads = weakref.WeakKeyDictionary() # for debugging
+        self._all_upload_statuses = weakref.WeakKeyDictionary()
+        self._recent_upload_statuses = []
+        self.stats_provider = stats_provider
+        if stats_provider:
+            stats_provider.register_producer(self)
+        self._counters = {"chk_upload_helper.upload_requests": 0,
+                          "chk_upload_helper.upload_already_present": 0,
+                          "chk_upload_helper.upload_need_upload": 0,
+                          "chk_upload_helper.resumes": 0,
+                          "chk_upload_helper.fetched_bytes": 0,
+                          "chk_upload_helper.encoded_bytes": 0,
+                          }
         service.MultiService.__init__(self)
 
+    def setServiceParent(self, parent):
+        service.MultiService.setServiceParent(self, parent)
+
     def log(self, *args, **kwargs):
         if 'facility' not in kwargs:
             kwargs['facility'] = "tahoe.helper"
         return self.parent.log(*args, **kwargs)
 
+    def count(self, key, value=1):
+        if self.stats_provider:
+            self.stats_provider.count(key, value)
+        self._counters[key] += value
+
+    def get_stats(self):
+        OLD = 86400*2 # 48hours
+        now = time.time()
+        inc_count = inc_size = inc_size_old = 0
+        enc_count = enc_size = enc_size_old = 0
+        inc = os.listdir(self._chk_incoming)
+        enc = os.listdir(self._chk_encoding)
+        for f in inc:
+            s = os.stat(os.path.join(self._chk_incoming, f))
+            size = s[stat.ST_SIZE]
+            mtime = s[stat.ST_MTIME]
+            inc_count += 1
+            inc_size += size
+            if now - mtime > OLD:
+                inc_size_old += size
+        for f in enc:
+            s = os.stat(os.path.join(self._chk_encoding, f))
+            size = s[stat.ST_SIZE]
+            mtime = s[stat.ST_MTIME]
+            enc_count += 1
+            enc_size += size
+            if now - mtime > OLD:
+                enc_size_old += size
+        stats = { 'chk_upload_helper.active_uploads': len(self._active_uploads),
+                  'chk_upload_helper.incoming_count': inc_count,
+                  'chk_upload_helper.incoming_size': inc_size,
+                  'chk_upload_helper.incoming_size_old': inc_size_old,
+                  'chk_upload_helper.encoding_count': enc_count,
+                  'chk_upload_helper.encoding_size': enc_size,
+                  'chk_upload_helper.encoding_size_old': enc_size_old,
+                  }
+        stats.update(self._counters)
+        return stats
+
     def remote_upload_chk(self, storage_index):
+        self.count("chk_upload_helper.upload_requests")
         r = upload.UploadResults()
         started = time.time()
         si_s = storage.si_b2a(storage_index)
@@ -498,9 +572,11 @@ class Helper(Referenceable, service.MultiService):
             r.timings['existence_check'] = elapsed
             if already_present:
                 # the necessary results are placed in the UploadResults
+                self.count("chk_upload_helper.upload_already_present")
                 self.log("file already found in grid", parent=lp)
                 return (r, None)
 
+            self.count("chk_upload_helper.upload_need_upload")
             # the file is not present in the grid, by which we mean there are
             # less than 'N' shares available.
             self.log("unable to find file in the grid", parent=lp,
@@ -516,6 +592,7 @@ class Helper(Referenceable, service.MultiService):
                                                   incoming_file, encoding_file,
                                                   r, lp)
                 self._active_uploads[storage_index] = uh
+                self._add_upload(uh)
             return uh.start()
         d.addCallback(_checked)
         def _err(f):
@@ -550,5 +627,21 @@ class Helper(Referenceable, service.MultiService):
         d.addCallback(_checked)
         return d
 
-    def upload_finished(self, storage_index):
+    def _add_upload(self, uh):
+        self._all_uploads[uh] = None
+        s = uh.get_upload_status()
+        self._all_upload_statuses[s] = None
+        self._recent_upload_statuses.append(s)
+        while len(self._recent_upload_statuses) > self.MAX_UPLOAD_STATUSES:
+            self._recent_upload_statuses.pop(0)
+
+    def upload_finished(self, storage_index, size):
+        # this is called with size=0 if the upload failed
+        self.count("chk_upload_helper.encoded_bytes", size)
+        uh = self._active_uploads[storage_index]
         del self._active_uploads[storage_index]
+        s = uh.get_upload_status()
+        s.set_active(False)
+
+    def get_all_upload_statuses(self):
+        return self._all_upload_statuses
