@@ -1,6 +1,5 @@
 import os, re, weakref, stat, struct, time
 from distutils.version import LooseVersion
-from itertools import chain
 
 from foolscap import Referenceable
 from twisted.application import service
@@ -9,7 +8,7 @@ from twisted.internet import defer
 from zope.interface import implements
 from allmydata.interfaces import RIStorageServer, RIBucketWriter, \
      RIBucketReader, IStorageBucketWriter, IStorageBucketReader, HASH_SIZE, \
-     BadWriteEnablerError, IStatsProducer
+     BadWriteEnablerError, IStatsProducer, FileTooLargeError
 from allmydata.util import base32, fileutil, idlib, mathutil, log
 from allmydata.util.assertutil import precondition, _assert
 import allmydata # for __version__
@@ -24,9 +23,8 @@ class DataTooLargeError(Exception):
 # storage/shares/$START/$STORAGEINDEX
 # storage/shares/$START/$STORAGEINDEX/$SHARENUM
 
-# Where "$START" denotes the first 14 bits worth of $STORAGEINDEX (that's 3
-# base-32 chars, but the last one has only 4 bits in it -- i.e. only 16 possible
-# chars in the last position).
+# Where "$START" denotes the first 10 bits worth of $STORAGEINDEX (that's 2
+# base-32 chars).
 
 # $SHARENUM matches this regex:
 NUM_RE=re.compile("^[0-9]+$")
@@ -700,10 +698,12 @@ class StorageServer(service.MultiService, Referenceable):
         self._active_writers = weakref.WeakKeyDictionary()
         lp = log.msg("StorageServer created, now measuring space..",
                      facility="tahoe.storage")
-        self.measure_size()
-        log.msg(format="space measurement done, consumed=%(consumed)d bytes",
-                consumed=self.consumed,
-                parent=lp, facility="tahoe.storage")
+        self.consumed = None
+        if self.sizelimit:
+            self.consumed = fileutil.du(self.sharedir)
+            log.msg(format="space measurement done, consumed=%(consumed)d bytes",
+                    consumed=self.consumed,
+                    parent=lp, facility="tahoe.storage")
 
     def log(self, *args, **kwargs):
         if "facility" not in kwargs:
@@ -726,15 +726,13 @@ class StorageServer(service.MultiService, Referenceable):
         fileutil.rm_dir(self.incomingdir)
 
     def get_stats(self):
-        return { 'storage_server.consumed': self.consumed,
-                 'storage_server.allocated': self.allocated_size(),
-               }
-
-    def measure_size(self):
-        self.consumed = fileutil.du(self.sharedir)
+        stats = { 'storage_server.allocated': self.allocated_size(), }
+        if self.consumed is not None:
+            stats['storage_server.consumed'] = self.consumed
+        return stats
 
     def allocated_size(self):
-        space = self.consumed
+        space = self.consumed or 0
         for bw in self._active_writers:
             space += bw.allocated_size()
         return space
@@ -752,6 +750,9 @@ class StorageServer(service.MultiService, Referenceable):
         alreadygot = set()
         bucketwriters = {} # k: shnum, v: BucketWriter
         si_dir = storage_index_to_dir(storage_index)
+        si_s = si_b2a(storage_index)
+
+        log.msg("storage: allocate_buckets %s" % si_s)
 
         # in this implementation, the lease information (including secrets)
         # goes into the share files themselves. It could also be put into a
@@ -770,8 +771,7 @@ class StorageServer(service.MultiService, Referenceable):
         # they asked about: this will save them a lot of work. Add or update
         # leases for all of them: if they want us to hold shares for this
         # file, they'll want us to hold leases for this file.
-        for (shnum, fn) in chain(self._get_bucket_shares(storage_index),
-                                 self._get_incoming_shares(storage_index)):
+        for (shnum, fn) in self._get_bucket_shares(storage_index):
             alreadygot.add(shnum)
             sf = ShareFile(fn)
             sf.add_or_renew_lease(lease_info)
@@ -783,9 +783,15 @@ class StorageServer(service.MultiService, Referenceable):
         for shnum in sharenums:
             incominghome = os.path.join(self.incomingdir, si_dir, "%d" % shnum)
             finalhome = os.path.join(self.sharedir, si_dir, "%d" % shnum)
-            if os.path.exists(incominghome) or os.path.exists(finalhome):
+            if os.path.exists(finalhome):
                 # great! we already have it. easy.
                 pass
+            elif os.path.exists(incominghome):
+                # Note that we don't create BucketWriters for shnums that
+                # have a partial share (in incoming/), so if a second upload
+                # occurs while the first is still in progress, the second
+                # uploader will use different storage servers.
+                pass
             elif no_limits or remaining_space >= space_per_bucket:
                 # ok! we need to create the new share file.
                 bw = BucketWriter(self, incominghome, finalhome,
@@ -863,14 +869,16 @@ class StorageServer(service.MultiService, Referenceable):
                 total_space_freed += filelen
         if not remaining_files:
             fileutil.rm_dir(storagedir)
-        self.consumed -= total_space_freed
+        if self.consumed is not None:
+            self.consumed -= total_space_freed
         if self.stats_provider:
             self.stats_provider.count('storage_server.bytes_freed', total_space_freed)
         if not found_buckets:
             raise IndexError("no such lease to cancel")
 
     def bucket_writer_closed(self, bw, consumed_size):
-        self.consumed += consumed_size
+        if self.consumed is not None:
+            self.consumed += consumed_size
         if self.stats_provider:
             self.stats_provider.count('storage_server.bytes_added', consumed_size)
         del self._active_writers[bw]
@@ -900,6 +908,8 @@ class StorageServer(service.MultiService, Referenceable):
             pass
 
     def remote_get_buckets(self, storage_index):
+        si_s = si_b2a(storage_index)
+        log.msg("storage: get_buckets %s" % si_s)
         bucketreaders = {} # k: sharenum, v: BucketReader
         for shnum, filename in self._get_bucket_shares(storage_index):
             bucketreaders[shnum] = BucketReader(filename)
@@ -927,6 +937,7 @@ class StorageServer(service.MultiService, Referenceable):
                                                test_and_write_vectors,
                                                read_vector):
         si_s = si_b2a(storage_index)
+        lp = log.msg("storage: slot_writev %s" % si_s)
         si_dir = storage_index_to_dir(storage_index)
         (write_enabler, renew_secret, cancel_secret) = secrets
         # shares exist if there is a file for them
@@ -1074,6 +1085,9 @@ class WriteBucketProxy:
         self._num_segments = num_segments
         self._nodeid = nodeid
 
+        if segment_size >= 2**32 or data_size >= 2**32:
+            raise FileTooLargeError("This file is too large to be uploaded (data_size).")
+
         effective_segments = mathutil.next_power_of_k(num_segments,2)
         self._segment_hash_size = (2*effective_segments - 1) * HASH_SIZE
         # how many share hashes are included in each share? This will be
@@ -1096,6 +1110,9 @@ class WriteBucketProxy:
         x += self._share_hash_size
         offsets['uri_extension'] = x
 
+        if x >= 2**32:
+            raise FileTooLargeError("This file is too large to be uploaded (offsets).")
+
         offset_data = struct.pack(">LLLLLLLLL",
                                   1, # version number
                                   segment_size,
@@ -1197,7 +1214,7 @@ class WriteBucketProxy:
         return self._rref.callRemote("close")
 
     def abort(self):
-        return self._rref.callRemote("abort")
+        return self._rref.callRemoteOnly("abort")
 
 class ReadBucketProxy:
     implements(IStorageBucketReader)
