@@ -2,12 +2,11 @@
 import os
 from twisted.trial import unittest
 from twisted.application import service
-from twisted.internet import reactor, defer
 from twisted.python import log
 
 import allmydata
 from allmydata import client, introducer
-from allmydata.util import base32
+from allmydata.util import base32, testutil
 from foolscap.eventual import flushEventualQueue
 
 class FakeIntroducerClient(introducer.IntroducerClient):
@@ -133,6 +132,10 @@ class Basic(unittest.TestCase):
         all_versions = allmydata.get_package_versions_string()
         self.failUnless("allmydata" in all_versions)
         log.msg("tahoe versions: %s" % all_versions)
+        # also test stats
+        stats = c.get_stats()
+        self.failUnless("node.uptime" in stats)
+        self.failUnless(isinstance(stats["node.uptime"], float))
 
 def flush_but_dont_ignore(res):
     d = flushEventualQueue()
@@ -141,7 +144,7 @@ def flush_but_dont_ignore(res):
     d.addCallback(_done)
     return d
 
-class Run(unittest.TestCase):
+class Run(unittest.TestCase, testutil.StallMixin):
 
     def setUp(self):
         self.sparent = service.MultiService()
@@ -159,11 +162,6 @@ class Run(unittest.TestCase):
         open(os.path.join(basedir, "suicide_prevention_hotline"), "w")
         c = client.Client(basedir)
 
-    def stall(self, res=None, delay=1):
-        d = defer.Deferred()
-        reactor.callLater(delay, d.callback, res)
-        return d
-
     def test_reloadable(self):
         basedir = "test_client.Run.test_reloadable"
         os.mkdir(basedir)
