@@ -4,11 +4,10 @@ from zope.interface import implements
 from twisted.trial import unittest
 from allmydata import uri, dirnode, upload
 from allmydata.interfaces import IURI, IClient, IMutableFileNode, \
-     INewDirectoryURI, IReadonlyNewDirectoryURI, IFileNode
+     INewDirectoryURI, IReadonlyNewDirectoryURI, IFileNode, ExistingChildError
 from allmydata.util import hashutil, testutil
 from allmydata.test.common import make_chk_file_uri, make_mutable_file_uri, \
      FakeDirectoryNode, create_chk_filenode
-from twisted.internet import defer, reactor
 
 # to test dirnode.py, we want to construct a tree of real DirectoryNodes that
 # contain pointers to fake files. We start with a fake MutableFileNode that
@@ -64,7 +63,7 @@ class FakeClient:
         return d
 
 
-class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
+class Dirnode(unittest.TestCase, testutil.ShouldFailMixin, testutil.StallMixin):
     def setUp(self):
         self.client = FakeClient()
 
@@ -139,12 +138,16 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                             ro_dn.set_uri, u"newchild", fileuri)
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
                             ro_dn.set_node, u"newchild", filenode)
+            self.shouldFail(dirnode.NotMutableError, "set_nodes ro", None,
+                            ro_dn.set_nodes, [ (u"newchild", filenode) ])
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
                             ro_dn.add_file, u"newchild", uploadable)
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
                             ro_dn.delete, u"child")
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
                             ro_dn.create_empty_directory, u"newchild")
+            self.shouldFail(dirnode.NotMutableError, "set_metadata_for ro", None,
+                            ro_dn.set_metadata_for, u"child", {})
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
                             ro_dn.move_child_to, u"child", rw_dn)
             self.shouldFail(dirnode.NotMutableError, "set_uri ro", None,
@@ -162,16 +165,12 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
     def failUnlessGreaterOrEqualThan(self, a, b):
         self.failUnless(a >= b, "%r should be >= %r" % (a, b))
 
-    def stall(self, res, delay=1.0):
-        d = defer.Deferred()
-        reactor.callLater(delay, d.callback, res)
-        return d
-
     def test_create(self):
         self.expected_manifest = []
 
         d = self.client.create_empty_dirnode()
         def _then(n):
+            # /
             self.failUnless(n.is_mutable())
             u = n.get_uri()
             self.failUnless(u)
@@ -187,13 +186,25 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
             d.addCallback(lambda res: n.has_child(u"missing"))
             d.addCallback(lambda res: self.failIf(res))
             fake_file_uri = make_mutable_file_uri()
+            other_file_uri = make_mutable_file_uri()
             m = Marker(fake_file_uri)
             ffu_v = m.get_verifier()
             assert isinstance(ffu_v, str)
             self.expected_manifest.append(ffu_v)
             d.addCallback(lambda res: n.set_uri(u"child", fake_file_uri))
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "set_uri-no",
+                                          "child 'child' already exists",
+                                          n.set_uri, u"child", other_file_uri,
+                                          overwrite=False))
+            # /
+            # /child = mutable
 
             d.addCallback(lambda res: n.create_empty_directory(u"subdir"))
+
+            # /
+            # /child = mutable
+            # /subdir = directory
             def _created(subdir):
                 self.failUnless(isinstance(subdir, FakeDirectoryNode))
                 self.subdir = subdir
@@ -202,6 +213,12 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                 self.expected_manifest.append(new_v)
             d.addCallback(_created)
 
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "mkdir-no",
+                                          "child 'subdir' already exists",
+                                          n.create_empty_directory, u"subdir",
+                                          overwrite=False))
+
             d.addCallback(lambda res: n.list())
             d.addCallback(lambda children:
                           self.failUnlessEqual(sorted(children.keys()),
@@ -213,6 +230,32 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                                      sorted(self.expected_manifest))
             d.addCallback(_check_manifest)
 
+            d.addCallback(lambda res: n.deep_stats())
+            def _check_deepstats(stats):
+                self.failUnless(isinstance(stats, dict))
+                expected = {"count-immutable-files": 0,
+                            "count-mutable-files": 1,
+                            "count-literal-files": 0,
+                            "count-files": 1,
+                            "count-directories": 2,
+                            "size-immutable-files": 0,
+                            "size-literal-files": 0,
+                            #"size-directories": 616, # varies
+                            #"largest-directory": 616,
+                            "largest-directory-children": 2,
+                            "largest-immutable-file": 0,
+                            }
+                for k,v in expected.iteritems():
+                    self.failUnlessEqual(stats[k], v,
+                                         "stats[%s] was %s, not %s" %
+                                         (k, stats[k], v))
+                self.failUnless(stats["size-directories"] > 500,
+                                stats["size-directories"])
+                self.failUnless(stats["largest-directory"] > 500,
+                                stats["largest-directory"])
+                self.failUnlessEqual(stats["size-files-histogram"], [])
+            d.addCallback(_check_deepstats)
+
             def _add_subsubdir(res):
                 return self.subdir.create_empty_directory(u"subsubdir")
             d.addCallback(_add_subsubdir)
@@ -257,6 +300,12 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
             # set_node + metadata
             # it should be possible to add a child without any metadata
             d.addCallback(lambda res: n.set_node(u"d2", n, {}))
+            d.addCallback(lambda res: self.client.create_empty_dirnode())
+            d.addCallback(lambda n2:
+                          self.shouldFail(ExistingChildError, "set_node-no",
+                                          "child 'd2' already exists",
+                                          n.set_node, u"d2", n2,
+                                          overwrite=False))
             d.addCallback(lambda res: n.get_metadata_for(u"d2"))
             d.addCallback(lambda metadata: self.failUnlessEqual(metadata, {}))
 
@@ -285,6 +334,16 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                                                    (u"e3", fake_file_uri,
                                                     {"key": "value"}),
                                                    ]))
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "set_children-no",
+                                          "child 'e1' already exists",
+                                          n.set_children,
+                                          [ (u"e1", other_file_uri),
+                                            (u"new", other_file_uri), ],
+                                          overwrite=False))
+            # and 'new' should not have been created
+            d.addCallback(lambda res: n.list())
+            d.addCallback(lambda children: self.failIf(u"new" in children))
             d.addCallback(lambda res: n.get_metadata_for(u"e1"))
             d.addCallback(lambda metadata:
                           self.failUnlessEqual(sorted(metadata.keys()),
@@ -305,6 +364,16 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                                                     (u"f3", n,
                                                      {"key": "value"}),
                                                     ]))
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "set_nodes-no",
+                                          "child 'f1' already exists",
+                                          n.set_nodes,
+                                          [ (u"f1", n),
+                                            (u"new", n), ],
+                                          overwrite=False))
+            # and 'new' should not have been created
+            d.addCallback(lambda res: n.list())
+            d.addCallback(lambda children: self.failIf(u"new" in children))
             d.addCallback(lambda res: n.get_metadata_for(u"f1"))
             d.addCallback(lambda metadata:
                           self.failUnlessEqual(sorted(metadata.keys()),
@@ -397,6 +466,13 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
             d.addCallback(lambda res: n.add_file(u"newfile", uploadable))
             d.addCallback(lambda newnode:
                           self.failUnless(IFileNode.providedBy(newnode)))
+            other_uploadable = upload.Data("some data", convergence="stuff")
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "add_file-no",
+                                          "child 'newfile' already exists",
+                                          n.add_file, u"newfile",
+                                          other_uploadable,
+                                          overwrite=False))
             d.addCallback(lambda res: n.list())
             d.addCallback(lambda children:
                           self.failUnlessEqual(sorted(children.keys()),
@@ -406,7 +482,6 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
                           self.failUnlessEqual(sorted(metadata.keys()),
                                                ["ctime", "mtime"]))
 
-            uploadable = upload.Data("some data", convergence="some convergence string")
             d.addCallback(lambda res: n.add_file(u"newfile-metadata",
                                                  uploadable,
                                                  {"key": "value"}))
@@ -420,6 +495,9 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
             d.addCallback(lambda res: n.create_empty_directory(u"subdir2"))
             def _created2(subdir2):
                 self.subdir2 = subdir2
+                # put something in the way, to make sure it gets overwritten
+                return subdir2.add_file(u"child", upload.Data("overwrite me",
+                                                              "converge"))
             d.addCallback(_created2)
 
             d.addCallback(lambda res:
@@ -432,6 +510,37 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
             d.addCallback(lambda children:
                           self.failUnlessEqual(sorted(children.keys()),
                                                sorted([u"child"])))
+            d.addCallback(lambda res: self.subdir2.get(u"child"))
+            d.addCallback(lambda child:
+                          self.failUnlessEqual(child.get_uri(),
+                                               fake_file_uri.to_string()))
+
+            # move it back, using new_child_name=
+            d.addCallback(lambda res:
+                          self.subdir2.move_child_to(u"child", n, u"newchild"))
+            d.addCallback(lambda res: n.list())
+            d.addCallback(lambda children:
+                          self.failUnlessEqual(sorted(children.keys()),
+                                               sorted([u"newchild", u"newfile",
+                                                       u"subdir2"])))
+            d.addCallback(lambda res: self.subdir2.list())
+            d.addCallback(lambda children:
+                          self.failUnlessEqual(sorted(children.keys()), []))
+
+            # now make sure that we honor overwrite=False
+            d.addCallback(lambda res:
+                          self.subdir2.set_uri(u"newchild", other_file_uri))
+
+            d.addCallback(lambda res:
+                          self.shouldFail(ExistingChildError, "move_child_to-no",
+                                          "child 'newchild' already exists",
+                                          n.move_child_to, u"newchild",
+                                          self.subdir2,
+                                          overwrite=False))
+            d.addCallback(lambda res: self.subdir2.get(u"newchild"))
+            d.addCallback(lambda child:
+                          self.failUnlessEqual(child.get_uri(),
+                                               other_file_uri.to_string()))
 
             return d
 
@@ -439,6 +548,49 @@ class Dirnode(unittest.TestCase, testutil.ShouldFailMixin):
 
         return d
 
+class DeepStats(unittest.TestCase):
+    def test_stats(self):
+        ds = dirnode.DeepStats()
+        ds.add("count-files")
+        ds.add("size-immutable-files", 123)
+        ds.histogram("size-files-histogram", 123)
+        ds.max("largest-directory", 444)
+
+        s = ds.get_results()
+        self.failUnlessEqual(s["count-files"], 1)
+        self.failUnlessEqual(s["size-immutable-files"], 123)
+        self.failUnlessEqual(s["largest-directory"], 444)
+        self.failUnlessEqual(s["count-literal-files"], 0)
+
+        ds.add("count-files")
+        ds.add("size-immutable-files", 321)
+        ds.histogram("size-files-histogram", 321)
+        ds.max("largest-directory", 2)
+
+        s = ds.get_results()
+        self.failUnlessEqual(s["count-files"], 2)
+        self.failUnlessEqual(s["size-immutable-files"], 444)
+        self.failUnlessEqual(s["largest-directory"], 444)
+        self.failUnlessEqual(s["count-literal-files"], 0)
+        self.failUnlessEqual(s["size-files-histogram"],
+                             [ (101, 316, 1), (317, 1000, 1) ])
+
+        ds = dirnode.DeepStats()
+        for i in range(1, 1100):
+            ds.histogram("size-files-histogram", i)
+        ds.histogram("size-files-histogram", 4*1000*1000*1000*1000) # 4TB
+        s = ds.get_results()
+        self.failUnlessEqual(s["size-files-histogram"],
+                             [ (1, 3, 3),
+                               (4, 10, 7),
+                               (11, 31, 21),
+                               (32, 100, 69),
+                               (101, 316, 216),
+                               (317, 1000, 684),
+                               (1001, 3162, 99),
+                               (3162277660169L, 10000000000000L, 1),
+                               ])
+
 
 netstring = hashutil.netstring
 split_netstring = dirnode.split_netstring
