@@ -58,6 +58,8 @@ class UploadResults(Copyable, RemoteCopy):
 # the extension changes size, we can change EXTENSION_SIZE to
 # allocate a more accurate amount of space.
 EXTENSION_SIZE = 1000
+# TODO: actual extensions are closer to 419 bytes, so we can probably lower
+# this.
 
 class PeerTracker:
     def __init__(self, peerid, storage_server,
@@ -155,7 +157,7 @@ class Tahoe2PeerSelector:
 
         peers = client.get_permuted_peers("storage", storage_index)
         if not peers:
-            raise encode.NotEnoughPeersError("client gave us zero peers")
+            raise encode.NotEnoughSharesError("client gave us zero peers")
 
         # figure out how much space to ask for
 
@@ -267,7 +269,7 @@ class Tahoe2PeerSelector:
                 if self.last_failure_msg:
                     msg += " (%s)" % (self.last_failure_msg,)
                 log.msg(msg, level=log.UNUSUAL, parent=self._log_parent)
-                raise encode.NotEnoughPeersError(msg)
+                raise encode.NotEnoughSharesError(msg)
             else:
                 # we placed enough to be happy, so we're done
                 if self._status:
@@ -1194,11 +1196,13 @@ class Uploader(service.MultiService):
     URI_LIT_SIZE_THRESHOLD = 55
     MAX_UPLOAD_STATUSES = 10
 
-    def __init__(self, helper_furl=None):
+    def __init__(self, helper_furl=None, stats_provider=None):
         self._helper_furl = helper_furl
+        self.stats_provider = stats_provider
         self._helper = None
-        self._all_uploads = weakref.WeakKeyDictionary()
-        self._recent_upload_status = []
+        self._all_uploads = weakref.WeakKeyDictionary() # for debugging
+        self._all_upload_statuses = weakref.WeakKeyDictionary()
+        self._recent_upload_statuses = []
         service.MultiService.__init__(self)
 
     def startService(self):
@@ -1229,16 +1233,18 @@ class Uploader(service.MultiService):
             precondition(isinstance(default_params, dict), default_params)
             precondition("max_segment_size" in default_params, default_params)
             uploadable.set_default_encoding_parameters(default_params)
+
+            if self.stats_provider:
+                self.stats_provider.count('uploader.files_uploaded', 1)
+                self.stats_provider.count('uploader.bytes_uploaded', size)
+
             if size <= self.URI_LIT_SIZE_THRESHOLD:
                 uploader = LiteralUploader(self.parent)
             elif self._helper:
                 uploader = AssistedUploader(self._helper)
             else:
                 uploader = self.uploader_class(self.parent)
-            self._all_uploads[uploader] = None
-            self._recent_upload_status.append(uploader.get_upload_status())
-            while len(self._recent_upload_status) > self.MAX_UPLOAD_STATUSES:
-                self._recent_upload_status.pop(0)
+            self._add_upload(uploader)
             return uploader.start(uploadable)
         d.addCallback(_got_size)
         def _done(res):
@@ -1247,10 +1253,14 @@ class Uploader(service.MultiService):
         d.addBoth(_done)
         return d
 
-    def list_all_uploads(self):
-        return self._all_uploads.keys()
-    def list_active_uploads(self):
-        return [u.get_upload_status() for u in self._all_uploads.keys()
-                if u.get_upload_status().get_active()]
-    def list_recent_uploads(self):
-        return self._recent_upload_status
+    def _add_upload(self, uploader):
+        s = uploader.get_upload_status()
+        self._all_uploads[uploader] = None
+        self._all_upload_statuses[s] = None
+        self._recent_upload_statuses.append(s)
+        while len(self._recent_upload_statuses) > self.MAX_UPLOAD_STATUSES:
+            self._recent_upload_statuses.pop(0)
+
+    def list_all_upload_statuses(self):
+        for us in self._all_upload_statuses:
+            yield us
