@@ -57,9 +57,16 @@ class PollMixin:
         if check_f():
             lc.stop()
 
+class StallMixin:
+    def stall(self, res=None, delay=1):
+        d = defer.Deferred()
+        reactor.callLater(delay, d.callback, res)
+        return d
+
 class ShouldFailMixin:
 
-    def shouldFail(self, expected_failure, which, substring, callable, *args, **kwargs):
+    def shouldFail(self, expected_failure, which, substring,
+                   callable, *args, **kwargs):
         assert substring is None or isinstance(substring, str)
         d = defer.maybeDeferred(callable, *args, **kwargs)
         def done(res):
@@ -67,8 +74,8 @@ class ShouldFailMixin:
                 res.trap(expected_failure)
                 if substring:
                     self.failUnless(substring in str(res),
-                                    "substring '%s' not in '%s'"
-                                    % (substring, str(res)))
+                                    "%s: substring '%s' not in '%s'"
+                                    % (which, substring, str(res)))
             else:
                 self.fail("%s was supposed to raise %s, not get '%s'" %
                           (which, expected_failure, res))
