@@ -29,7 +29,7 @@ class IntroducerRoot(rend.Page):
         res["subscription_summary"] = subscription_summary
 
         announcement_summary = {}
-        for ann in i.get_announcements():
+        for (ann,when) in i.get_announcements().values():
             (furl, service_name, ri_name, nickname, ver, oldest) = ann
             if service_name not in announcement_summary:
                 announcement_summary[service_name] = 0
@@ -48,7 +48,7 @@ class IntroducerRoot(rend.Page):
     def render_announcement_summary(self, ctx, data):
         i = IClient(ctx).getServiceNamed("introducer")
         services = {}
-        for ann in i.get_announcements():
+        for (ann,when) in i.get_announcements().values():
             (furl, service_name, ri_name, nickname, ver, oldest) = ann
             if service_name not in services:
                 services[service_name] = 0
@@ -69,7 +69,7 @@ class IntroducerRoot(rend.Page):
     def data_services(self, ctx, data):
         i = IClient(ctx).getServiceNamed("introducer")
         ann = [(since,a)
-               for (a,since) in i.get_announcements().items()
+               for (a,since) in i.get_announcements().values()
                if a[1] != "stub_client"]
         ann.sort(lambda a,b: cmp( (a[1][1], a), (b[1][1], b) ) )
         return ann
@@ -78,8 +78,7 @@ class IntroducerRoot(rend.Page):
         (furl, service_name, ri_name, nickname, ver, oldest) = announcement
         sr = SturdyRef(furl)
         nodeid = sr.tubID
-        advertised = [loc.split(":")[0] for loc in sr.locationHints
-                      if not loc.startswith("127.0.0.1:")]
+        advertised = self.show_location_hints(sr)
         ctx.fillSlots("peerid", "%s %s" % (nodeid, nickname))
         ctx.fillSlots("advertised", " ".join(advertised))
         ctx.fillSlots("connected", "?")
@@ -94,7 +93,7 @@ class IntroducerRoot(rend.Page):
         i = IClient(ctx).getServiceNamed("introducer")
         # use the "stub_client" announcements to get information per nodeid
         clients = {}
-        for ann in i.get_announcements():
+        for (ann,when) in i.get_announcements().values():
             if ann[1] != "stub_client":
                 continue
             (furl, service_name, ri_name, nickname, ver, oldest) = ann
@@ -124,8 +123,7 @@ class IntroducerRoot(rend.Page):
         # if the subscriber didn't do Tub.setLocation, nodeid will be None
         nodeid = sr.tubID or "?"
         ctx.fillSlots("peerid", "%s %s" % (nodeid, nickname))
-        advertised = [loc.split(":")[0] for loc in sr.locationHints
-                      if not loc.startswith("127.0.0.1:")]
+        advertised = self.show_location_hints(sr)
         ctx.fillSlots("advertised", " ".join(advertised))
         remote_host = rref.tracker.broker.transport.getPeer()
         if isinstance(remote_host, address.IPv4Address):
@@ -141,4 +139,21 @@ class IntroducerRoot(rend.Page):
         ctx.fillSlots("service_name", service_name)
         return ctx.tag
 
+    def show_location_hints(self, sr, ignore_localhost=True):
+        advertised = []
+        for hint in sr.locationHints:
+            if isinstance(hint, str):
+                # Foolscap-0.2.5 and earlier used strings in .locationHints
+                if ignore_localhost and hint.startswith("127.0.0.1"):
+                    continue
+                advertised.append(hint.split(":")[0])
+            else:
+                # Foolscap-0.2.6 and later use tuples of ("ipv4", host, port)
+                if hint[0] == "ipv4":
+                    host = hint[1]
+                if ignore_localhost and host == "127.0.0.1":
+                    continue
+                advertised.append(hint[1])
+        return advertised
+
 
