@@ -1,12 +1,13 @@
 
-import time
+import time, pprint, itertools
+import simplejson
 from twisted.internet import defer
-from nevow import rend, tags as T
+from nevow import rend, inevow, tags as T
 from allmydata.util import base32, idlib
 from allmydata.web.common import IClient, getxmlfile, abbreviate_time, \
-     abbreviate_rate
+     abbreviate_rate, abbreviate_size, get_arg
 from allmydata.interfaces import IUploadStatus, IDownloadStatus, \
-     IPublishStatus, IRetrieveStatus
+     IPublishStatus, IRetrieveStatus, IServermapUpdaterStatus
 
 def plural(sequence_or_length):
     if isinstance(sequence_or_length, int):
@@ -142,13 +143,9 @@ class UploadResultsRendererMixin(RateAndTimeMixin):
         d = self.upload_results()
         def _convert(r):
             file_size = r.file_size
-            if file_size is None:
-                return None
             time1 = r.timings.get("cumulative_encoding")
-            if time1 is None:
-                return None
             time2 = r.timings.get("cumulative_sending")
-            if time2 is None:
+            if (file_size is None or time1 is None or time2 is None):
                 return None
             try:
                 return 1.0 * file_size / (time1+time2)
@@ -161,10 +158,8 @@ class UploadResultsRendererMixin(RateAndTimeMixin):
         d = self.upload_results()
         def _convert(r):
             fetch_size = r.ciphertext_fetched
-            if fetch_size is None:
-                return None
             time = r.timings.get("cumulative_fetch")
-            if time is None:
+            if (fetch_size is None or time is None):
                 return None
             try:
                 return 1.0 * fetch_size / time
@@ -211,7 +206,7 @@ class UploadStatusPage(UploadResultsRendererMixin, rend.Page):
     def render_total_size(self, ctx, data):
         size = data.get_size()
         if size is None:
-            size = "(unknown)"
+            return "(unknown)"
         return size
 
     def render_progress_hash(self, ctx, data):
@@ -313,6 +308,9 @@ class DownloadResultsRendererMixin(RateAndTimeMixin):
     def data_time_cumulative_decrypt(self, ctx, data):
         return self._get_time("cumulative_decrypt")
 
+    def data_time_paused(self, ctx, data):
+        return self._get_time("paused")
+
     def _get_rate(self, name):
         d = self.download_results()
         def _convert(r):
@@ -396,7 +394,7 @@ class DownloadStatusPage(DownloadResultsRendererMixin, rend.Page):
     def render_total_size(self, ctx, data):
         size = data.get_size()
         if size is None:
-            size = "(unknown)"
+            return "(unknown)"
         return size
 
     def render_progress(self, ctx, data):
@@ -448,10 +446,6 @@ class RetrieveStatusPage(rend.Page, RateAndTimeMixin):
         k, n = data.get_encoding()
         return ctx.tag["Encoding: %s of %s" % (k, n)]
 
-    def render_search_distance(self, ctx, data):
-        d = data.get_search_distance()
-        return ctx.tag["Search Distance: %s peer%s" % (d, plural(d))]
-
     def render_problems(self, ctx, data):
         problems = data.problems
         if not problems:
@@ -477,17 +471,11 @@ class RetrieveStatusPage(rend.Page, RateAndTimeMixin):
     def data_rate_total(self, ctx, data):
         return self._get_rate(data, "total")
 
-    def data_time_peer_selection(self, ctx, data):
-        return self.retrieve_status.timings.get("peer_selection")
-
     def data_time_fetch(self, ctx, data):
         return self.retrieve_status.timings.get("fetch")
     def data_rate_fetch(self, ctx, data):
         return self._get_rate(data, "fetch")
 
-    def data_time_cumulative_verify(self, ctx, data):
-        return self.retrieve_status.timings.get("cumulative_verify")
-
     def data_time_decode(self, ctx, data):
         return self.retrieve_status.timings.get("decode")
     def data_rate_decode(self, ctx, data):
@@ -552,19 +540,16 @@ class PublishStatusPage(rend.Page, RateAndTimeMixin):
         k, n = data.get_encoding()
         return ctx.tag["Encoding: %s of %s" % (k, n)]
 
-    def render_peers_queried(self, ctx, data):
-        return ctx.tag["Peers Queried: ", data.peers_queried]
-
     def render_sharemap(self, ctx, data):
-        sharemap = data.sharemap
-        if sharemap is None:
+        servermap = data.get_servermap()
+        if servermap is None:
             return ctx.tag["None"]
         l = T.ul()
+        sharemap = servermap.make_sharemap()
         for shnum in sorted(sharemap.keys()):
             l[T.li["%d -> Placed on " % shnum,
                    ", ".join(["[%s]" % idlib.shortnodeid_b2a(peerid)
-                              for (peerid,seqnum,root_hash)
-                              in sharemap[shnum]])]]
+                              for peerid in sharemap[shnum]])]]
         return ctx.tag["Sharemap:", l]
 
     def render_problems(self, ctx, data):
@@ -595,21 +580,6 @@ class PublishStatusPage(rend.Page, RateAndTimeMixin):
     def data_time_setup(self, ctx, data):
         return self.publish_status.timings.get("setup")
 
-    def data_time_query(self, ctx, data):
-        return self.publish_status.timings.get("query")
-
-    def data_time_privkey(self, ctx, data):
-        return self.publish_status.timings.get("privkey")
-
-    def data_time_privkey_fetch(self, ctx, data):
-        return self.publish_status.timings.get("privkey_fetch")
-    def render_privkey_from(self, ctx, data):
-        peerid = data.privkey_from
-        if peerid:
-            return " (got from [%s])" % idlib.shortnodeid_b2a(peerid)
-        else:
-            return ""
-
     def data_time_encrypt(self, ctx, data):
         return self.publish_status.timings.get("encrypt")
     def data_rate_encrypt(self, ctx, data):
@@ -632,44 +602,197 @@ class PublishStatusPage(rend.Page, RateAndTimeMixin):
     def data_rate_push(self, ctx, data):
         return self._get_rate(data, "push")
 
-    def data_initial_read_size(self, ctx, data):
-        return self.publish_status.initial_read_size
+    def render_server_timings(self, ctx, data):
+        per_server = self.publish_status.timings.get("send_per_server")
+        if not per_server:
+            return ""
+        l = T.ul()
+        for peerid in sorted(per_server.keys()):
+            peerid_s = idlib.shortnodeid_b2a(peerid)
+            times_s = ", ".join([self.render_time(None, t)
+                                 for t in per_server[peerid]])
+            l[T.li["[%s]: %s" % (peerid_s, times_s)]]
+        return T.li["Per-Server Response Times: ", l]
+
+class MapupdateStatusPage(rend.Page, RateAndTimeMixin):
+    docFactory = getxmlfile("map-update-status.xhtml")
+
+    def __init__(self, data):
+        rend.Page.__init__(self, data)
+        self.update_status = data
+
+    def render_started(self, ctx, data):
+        TIME_FORMAT = "%H:%M:%S %d-%b-%Y"
+        started_s = time.strftime(TIME_FORMAT,
+                                  time.localtime(data.get_started()))
+        return started_s
+
+    def render_finished(self, ctx, data):
+        when = data.get_finished()
+        if not when:
+            return "not yet"
+        TIME_FORMAT = "%H:%M:%S %d-%b-%Y"
+        started_s = time.strftime(TIME_FORMAT,
+                                  time.localtime(data.get_finished()))
+        return started_s
+
+    def render_si(self, ctx, data):
+        si_s = base32.b2a_or_none(data.get_storage_index())
+        if si_s is None:
+            si_s = "(None)"
+        return si_s
+
+    def render_helper(self, ctx, data):
+        return {True: "Yes",
+                False: "No"}[data.using_helper()]
+
+    def render_progress(self, ctx, data):
+        progress = data.get_progress()
+        # TODO: make an ascii-art bar
+        return "%.1f%%" % (100.0 * progress)
+
+    def render_status(self, ctx, data):
+        return data.get_status()
+
+    def render_problems(self, ctx, data):
+        problems = data.problems
+        if not problems:
+            return ""
+        l = T.ul()
+        for peerid in sorted(problems.keys()):
+            peerid_s = idlib.shortnodeid_b2a(peerid)
+            l[T.li["[%s]: %s" % (peerid_s, problems[peerid])]]
+        return ctx.tag["Server Problems:", l]
+
+    def render_privkey_from(self, ctx, data):
+        peerid = data.get_privkey_from()
+        if peerid:
+            return ctx.tag["Got privkey from: [%s]"
+                           % idlib.shortnodeid_b2a(peerid)]
+        else:
+            return ""
+
+    def data_time_total(self, ctx, data):
+        return self.update_status.timings.get("total")
+
+    def data_time_initial_queries(self, ctx, data):
+        return self.update_status.timings.get("initial_queries")
+
+    def data_time_cumulative_verify(self, ctx, data):
+        return self.update_status.timings.get("cumulative_verify")
 
     def render_server_timings(self, ctx, data):
-        per_server = self.publish_status.timings.get("per_server")
+        per_server = self.update_status.timings.get("per_server")
         if not per_server:
             return ""
         l = T.ul()
         for peerid in sorted(per_server.keys()):
             peerid_s = idlib.shortnodeid_b2a(peerid)
             times = []
-            for op,t in per_server[peerid]:
-                if op == "read":
-                    times.append( "(" + self.render_time(None, t) + ")" )
-                else:
+            for op,started,t in per_server[peerid]:
+                #times.append("%s/%.4fs/%s/%s" % (op,
+                #                              started,
+                #                              self.render_time(None, started - self.update_status.get_started()),
+                #                              self.render_time(None,t)))
+                if op == "query":
                     times.append( self.render_time(None, t) )
+                elif op == "late":
+                    times.append( "late(" + self.render_time(None, t) + ")" )
+                else:
+                    times.append( "privkey(" + self.render_time(None, t) + ")" )
             times_s = ", ".join(times)
             l[T.li["[%s]: %s" % (peerid_s, times_s)]]
         return T.li["Per-Server Response Times: ", l]
 
+    def render_timing_chart(self, ctx, data):
+        imageurl = self._timing_chart()
+        return ctx.tag[imageurl]
+
+    def _timing_chart(self):
+        started = self.update_status.get_started()
+        total = self.update_status.timings.get("total")
+        per_server = self.update_status.timings.get("per_server")
+        base = "http://chart.apis.google.com/chart?"
+        pieces = ["cht=bhs", "chs=400x300"]
+        pieces.append("chco=ffffff,4d89f9,c6d9fd") # colors
+        data0 = []
+        data1 = []
+        data2 = []
+        peerids_s = []
+        top_abs = started
+        # we sort the queries by the time at which we sent the first request
+        sorttable = [ (times[0][1], peerid)
+                      for peerid, times in per_server.items() ]
+        sorttable.sort()
+        peerids = [t[1] for t in sorttable]
+
+        for peerid in peerids:
+            times = per_server[peerid]
+            peerid_s = idlib.shortnodeid_b2a(peerid)
+            peerids_s.append(peerid_s)
+            # for servermap updates, there are either one or two queries per
+            # peer. The second (if present) is to get the privkey.
+            op,q_started,q_elapsed = times[0]
+            data0.append("%.3f" % (q_started-started))
+            data1.append("%.3f" % q_elapsed)
+            top_abs = max(top_abs, q_started+q_elapsed)
+            if len(times) > 1:
+                op,p_started,p_elapsed = times[0]
+                data2.append("%.3f" % p_elapsed)
+                top_abs = max(top_abs, p_started+p_elapsed)
+            else:
+                data2.append("0.0")
+        finished = self.update_status.get_finished()
+        if finished:
+            top_abs = max(top_abs, finished)
+        top_rel = top_abs - started
+        chd = "chd=t:" + "|".join([",".join(data0),
+                                   ",".join(data1),
+                                   ",".join(data2)])
+        pieces.append(chd)
+        chds = "chds=0,%0.3f" % top_rel
+        pieces.append(chds)
+        pieces.append("chxt=x,y")
+        pieces.append("chxr=0,0.0,%0.3f" % top_rel)
+        pieces.append("chxl=1:|" + "|".join(reversed(peerids_s)))
+        # use up to 10 grid lines, at decimal multiples.
+        # mathutil.next_power_of_k doesn't handle numbers smaller than one,
+        # unfortunately.
+        #pieces.append("chg="
+
+        if total is not None:
+            finished_f = 1.0 * total / top_rel
+            pieces.append("chm=r,FF0000,0,%0.3f,%0.3f" % (finished_f,
+                                                          finished_f+0.01))
+        url = base + "&".join(pieces)
+        return T.img(src=url, align="right", float="right")
+
 
 class Status(rend.Page):
     docFactory = getxmlfile("status.xhtml")
     addSlash = True
 
+    def _get_all_statuses(self, client):
+        return itertools.chain(client.list_all_upload_statuses(),
+                               client.list_all_download_statuses(),
+                               client.list_all_mapupdate_statuses(),
+                               client.list_all_publish_statuses(),
+                               client.list_all_retrieve_statuses(),
+                               client.list_all_helper_statuses(),
+                               )
+
     def data_active_operations(self, ctx, data):
-        active =  (IClient(ctx).list_active_uploads() +
-                   IClient(ctx).list_active_downloads() +
-                   IClient(ctx).list_active_publish() +
-                   IClient(ctx).list_active_retrieve())
+        client = IClient(ctx)
+        active = [s
+                  for s in self._get_all_statuses(client)
+                  if s.get_active()]
         return active
 
     def data_recent_operations(self, ctx, data):
-        recent = [o for o in (IClient(ctx).list_recent_uploads() +
-                              IClient(ctx).list_recent_downloads() +
-                              IClient(ctx).list_recent_publish() +
-                              IClient(ctx).list_recent_retrieve())
-                  if not o.get_active()]
+        client = IClient(ctx)
+        recent = [s
+                  for s in self._get_all_statuses(client)
+                  if not s.get_active()]
         recent.sort(lambda a,b: cmp(a.get_started(), b.get_started()))
         recent.reverse()
         return recent
@@ -692,6 +815,8 @@ class Status(rend.Page):
         size = s.get_size()
         if size is None:
             size = "(unknown)"
+        elif isinstance(size, (int, long, float)):
+            size = abbreviate_size(size)
         ctx.fillSlots("total_size", size)
 
         progress = data.get_progress()
@@ -713,11 +838,15 @@ class Status(rend.Page):
             link = "publish-%d" % data.get_counter()
             ctx.fillSlots("type", "publish")
             ctx.fillSlots("progress", "%.1f%%" % (100.0 * progress))
-        else:
-            assert IRetrieveStatus.providedBy(data)
+        elif IRetrieveStatus.providedBy(data):
             ctx.fillSlots("type", "retrieve")
             link = "retrieve-%d" % data.get_counter()
             ctx.fillSlots("progress", "%.1f%%" % (100.0 * progress))
+        else:
+            assert IServermapUpdaterStatus.providedBy(data)
+            ctx.fillSlots("type", "mapupdate %s" % data.get_mode())
+            link = "mapupdate-%d" % data.get_counter()
+            ctx.fillSlots("progress", "%.1f%%" % (100.0 * progress))
         ctx.fillSlots("status", T.a(href=link)[s.get_status()])
         return ctx.tag
 
@@ -726,32 +855,123 @@ class Status(rend.Page):
         stype,count_s = name.split("-")
         count = int(count_s)
         if stype == "up":
-            for s in client.list_recent_uploads():
-                if s.get_counter() == count:
-                    return UploadStatusPage(s)
-            for s in client.list_all_uploads():
+            for s in itertools.chain(client.list_all_upload_statuses(),
+                                     client.list_all_helper_statuses()):
+                # immutable-upload helpers use the same status object as a
+                # regular immutable-upload
                 if s.get_counter() == count:
                     return UploadStatusPage(s)
         if stype == "down":
-            for s in client.list_recent_downloads():
+            for s in client.list_all_download_statuses():
                 if s.get_counter() == count:
                     return DownloadStatusPage(s)
-            for s in client.list_all_downloads():
+        if stype == "mapupdate":
+            for s in client.list_all_mapupdate_statuses():
                 if s.get_counter() == count:
-                    return DownloadStatusPage(s)
+                    return MapupdateStatusPage(s)
         if stype == "publish":
-            for s in client.list_recent_publish():
-                if s.get_counter() == count:
-                    return PublishStatusPage(s)
-            for s in client.list_all_publish():
+            for s in client.list_all_publish_statuses():
                 if s.get_counter() == count:
                     return PublishStatusPage(s)
         if stype == "retrieve":
-            for s in client.list_recent_retrieve():
-                if s.get_counter() == count:
-                    return RetrieveStatusPage(s)
-            for s in client.list_all_retrieve():
+            for s in client.list_all_retrieve_statuses():
                 if s.get_counter() == count:
                     return RetrieveStatusPage(s)
 
 
+class HelperStatus(rend.Page):
+    docFactory = getxmlfile("helper.xhtml")
+
+    def renderHTTP(self, ctx):
+        t = get_arg(inevow.IRequest(ctx), "t")
+        if t == "json":
+            return self.render_JSON(ctx)
+        # is there a better way to provide 'data' to all rendering methods?
+        helper = IClient(ctx).getServiceNamed("helper")
+        self.original = helper.get_stats()
+        return rend.Page.renderHTTP(self, ctx)
+
+    def render_JSON(self, ctx):
+        try:
+            h = IClient(ctx).getServiceNamed("helper")
+        except KeyError:
+            return simplejson.dumps({})
+
+        stats = h.get_stats()
+        return simplejson.dumps(stats, indent=1)
+
+    def render_active_uploads(self, ctx, data):
+        return data["chk_upload_helper.active_uploads"]
+
+    def render_incoming(self, ctx, data):
+        return "%d bytes in %d files" % (data["chk_upload_helper.incoming_size"],
+                                         data["chk_upload_helper.incoming_count"])
+
+    def render_encoding(self, ctx, data):
+        return "%d bytes in %d files" % (data["chk_upload_helper.encoding_size"],
+                                         data["chk_upload_helper.encoding_count"])
+
+    def render_upload_requests(self, ctx, data):
+        return str(data["chk_upload_helper.upload_requests"])
+
+    def render_upload_already_present(self, ctx, data):
+        return str(data["chk_upload_helper.upload_already_present"])
+
+    def render_upload_need_upload(self, ctx, data):
+        return str(data["chk_upload_helper.upload_need_upload"])
+
+    def render_upload_bytes_fetched(self, ctx, data):
+        return str(data["chk_upload_helper.fetched_bytes"])
+
+    def render_upload_bytes_encoded(self, ctx, data):
+        return str(data["chk_upload_helper.encoded_bytes"])
+
+
+class Statistics(rend.Page):
+    docFactory = getxmlfile("statistics.xhtml")
+
+    def renderHTTP(self, ctx):
+        provider = IClient(ctx).stats_provider
+        stats = {'stats': {}, 'counters': {}}
+        if provider:
+            stats = provider.get_stats()
+        t = get_arg(inevow.IRequest(ctx), "t")
+        if t == "json":
+            return simplejson.dumps(stats, indent=1)
+        # is there a better way to provide 'data' to all rendering methods?
+        self.original = stats
+        return rend.Page.renderHTTP(self, ctx)
+
+    def render_load_average(self, ctx, data):
+        return str(data["stats"].get("load_monitor.avg_load"))
+
+    def render_peak_load(self, ctx, data):
+        return str(data["stats"].get("load_monitor.max_load"))
+
+    def render_uploads(self, ctx, data):
+        files = data["counters"].get("uploader.files_uploaded", 0)
+        bytes = data["counters"].get("uploader.bytes_uploaded", 0)
+        return ("%s files / %s bytes (%s)" %
+                (files, bytes, abbreviate_size(bytes)))
+
+    def render_downloads(self, ctx, data):
+        files = data["counters"].get("downloader.files_downloaded", 0)
+        bytes = data["counters"].get("downloader.bytes_downloaded", 0)
+        return ("%s files / %s bytes (%s)" %
+                (files, bytes, abbreviate_size(bytes)))
+
+    def render_publishes(self, ctx, data):
+        files = data["counters"].get("mutable.files_published")
+        bytes = data["counters"].get("mutable.bytes_published", 0)
+        return "%s files / %s bytes (%s)" % (files, bytes,
+                                             abbreviate_size(bytes))
+
+    def render_retrieves(self, ctx, data):
+        files = data["counters"].get("mutable.files_retrieved")
+        bytes = data["counters"].get("mutable.bytes_retrieved", 0)
+        return "%s files / %s bytes (%s)" % (files, bytes,
+                                             abbreviate_size(bytes))
+
+    def render_raw(self, ctx, data):
+        raw = pprint.pformat(data)
+        return ctx.tag[raw]
